"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChartHelper = void 0;
const d3Scale = __importStar(require("d3-scale"));
const lodash_1 = require("lodash");
const metrics_1 = require("./metrics");
/**
 * Creates a linear scale for the provided values.
 * The domain is calculated from the values unless `range` is given.
 * Also returns the values to show on the axis.
 *
 * If `values` is empty, then the scale and ticks will be undefined.
 */
const getYScaleAndTicks = (range, values, height, yTickCount) => {
    if (Object.keys(values).length === 0) {
        return {};
    }
    const [yMin, yMax] = range !== null && range !== void 0 ? range : calculateYLimits(values);
    if (yMin === Number.POSITIVE_INFINITY) {
        // This means that all arrays in `values` are empty
        return {};
    }
    const yScale = d3Scale
        .scaleLinear()
        .domain([yMin, yMax])
        .range([
        height - metrics_1.DOUBLE_AXIS_CHART_METRICS.BOTTOM_PADDING,
        metrics_1.DOUBLE_AXIS_CHART_METRICS.TOP_PADDING + metrics_1.LINE_CHART_TEXT_SIZE,
    ]);
    const tickValues = getAxisValues(yMin, yMax, yTickCount);
    return { tickValues, yScale };
};
/**
 * Creates a linear scale for the provided grouped values.
 * The domain is calculated from the values is given.
 * Also returns the values to show on the axis.
 *
 * If `groupedValues` is empty, then the scale and ticks will be undefined.
 */
const getGroupedYScaleAndTicks = (groupedValues, height, yTickCount) => {
    if (Object.keys(groupedValues).length === 0) {
        return {};
    }
    const [yMin, yMax] = calculateGroupedLimits(groupedValues);
    const yScale = d3Scale
        .scaleLinear()
        .domain([yMin, yMax])
        .range([
        height - metrics_1.DOUBLE_AXIS_CHART_METRICS.BOTTOM_PADDING,
        metrics_1.DOUBLE_AXIS_CHART_METRICS.TOP_PADDING + metrics_1.LINE_CHART_TEXT_SIZE,
    ]);
    const tickValues = getAxisValues(yMin, yMax, yTickCount);
    return { tickValues, yScale };
};
/**
 * Returns the bounds for the charted based on the
 * minimum and maximum values found within the
 * map of values.
 *
 * If all parameters have empty arrays or `undefined` for values,
 * `[Infinity, -Infinity]` will be returned.
 */
const calculateYLimits = (values) => {
    let min = Number.POSITIVE_INFINITY;
    let max = Number.NEGATIVE_INFINITY;
    Object.values(values).forEach((parameterValues) => parameterValues.forEach(({ value }) => {
        if (value === undefined) {
            return;
        }
        if (value < min) {
            min = value;
        }
        if (value > max) {
            max = value;
        }
    }));
    return min !== Number.POSITIVE_INFINITY
        ? calculateBounds(Math.min(0, min), max)
        : [min, max];
};
/**
 * Calculates the limits from grouped values based on the minimum of the
 * negativeSums and the maximum of the positiveSums.
 */
const calculateGroupedLimits = (groupedValues) => {
    let min = Number.POSITIVE_INFINITY;
    let max = Number.NEGATIVE_INFINITY;
    Object.values(groupedValues).forEach(({ positiveSum, negativeSum }) => {
        if (negativeSum < min) {
            min = negativeSum;
        }
        if (positiveSum > max) {
            max = positiveSum;
        }
    });
    return min !== Number.POSITIVE_INFINITY
        ? calculateBounds(Math.min(0, min), max)
        : [min, max];
};
/**
 * Calculates the boundary values for the data such as the divider lines
 * show round values.
 */
const calculateBounds = (actualMin, actualMax) => {
    const roundMin = Math.floor(actualMin);
    const roundMax = Math.ceil(actualMax);
    const roundTarget = getRoundTarget(roundMax - roundMin);
    const max = roundTo(roundMax, roundTarget);
    const min = roundTo(roundMin, roundTarget);
    return [min, max];
};
/**
 * Rounds the passed number up to a multiple of `target`.
 * If `number` is negative, it will be rounded down.
 */
function roundTo(num, target) {
    const abs = Math.abs(num);
    const sign = num < 0 ? -1 : 1;
    return sign * (abs + ((target - (abs % target)) % target));
}
/**
 * Calculates the optimal rounding target so that the dividers represent
 * round numbers, but prevents most of the chart being empty.
 */
function getRoundTarget(intervalSize) {
    const numStr = Math.abs(intervalSize).toString();
    const mostSignificantDigit = Number.parseInt(numStr[0], 10);
    /** The distance between two divider lines. */
    let step = 1;
    // eslint-disable-next-line unicorn/prefer-ternary
    if (mostSignificantDigit < 4) {
        /*
        NOTE: for numbers 1xxxx - 3xxxx: step should be 1xxx
        Examples: 3 -> 1, 256 -> 10, 3489 -> 100
        */
        step = Math.pow(10, Math.max(0, numStr.length - 2));
    }
    else {
        /*
        NOTE: for numbers 4xxxx - 9xxxx: step should be 5xxx
        Examples: 4 -> 1, 401 -> 50, 4242 -> 500
        */
        step = Math.ceil(Math.pow(10, numStr.length - 1) / 2);
    }
    return step * 4;
}
/**
 * Validates and generates axis tick values.
 * If the ticksNumber is less or equal to 2 then
 * the DEFAULT_AXIS_TICKS_NUMBER is used and a warning message is shown.
 *
 * @param min interval range minimum value
 * @param max interval range maximum value
 * @param ticks ticks number to generate
 */
const getAxisValues = (min, max, ticks) => {
    let ticksNumber = ticks;
    if (ticksNumber < 2) {
        console.warn(`Tick number must be at least 2. Falling back to default ticks number: ${metrics_1.DEFAULT_AXIS_TICKS_NUMBER}.`);
        ticksNumber = metrics_1.DEFAULT_AXIS_TICKS_NUMBER;
    }
    const tickValues = (0, lodash_1.range)(min, max + 1, (max - min) / (ticksNumber - 1));
    if (tickValues.length < ticksNumber && !tickValues.includes(max)) {
        tickValues.push(max);
    }
    return tickValues;
};
/**
 * Returns the value of each parameter at the specified timestamp.
 * If a value cannot be found, `undefined` is returned for that parameter.
 */
const findValuesByTimestamp = (values, selectedTimestamp) => {
    return Object.fromEntries(Object.entries(values).map(([parameter, paramValues]) => {
        var _a;
        return [
            parameter,
            (_a = paramValues.find((entry) => entry.timestamp === selectedTimestamp)) === null || _a === void 0 ? void 0 : _a.value,
        ];
    }));
};
/**
 * Groups parameter values into a single map to help displaying stacked values.
 *
 * @param activeParameters  list of parameters that should be shown
 * @param values  map containing the values of each parameter
 */
const groupValues = (activeParameters, values) => {
    const groupedValues = {};
    activeParameters.forEach((parameter) => {
        const parameterValues = values[parameter];
        if (!parameterValues) {
            return;
        }
        parameterValues.forEach(({ timestamp, value }) => {
            if (value === undefined) {
                return;
            }
            if (!groupedValues[timestamp]) {
                groupedValues[timestamp] = {
                    timestamp,
                    positiveSum: 0,
                    negativeSum: 0,
                };
            }
            groupedValues[timestamp][parameter] = value;
            if (value > 0) {
                groupedValues[timestamp].positiveSum += value;
            }
            else {
                groupedValues[timestamp].negativeSum += value;
            }
        });
    });
    return groupedValues;
};
exports.ChartHelper = {
    getYScaleAndTicks,
    getGroupedYScaleAndTicks,
    calculateYLimits,
    getAxisValues,
    findValuesByTimestamp,
    groupValues,
};
