import { FC } from "react";
import { Colors, TestProps } from "@cs-bosch-medusa/ui";
import type { ChartSelectionProps } from "./common-props";
import { ChartValue } from "./types";
export type DoubleAxisLineChartProps = TestProps & {
    /** Width of the chart. */
    width: number;
    /** Height of the chart. */
    height: number;
    /** The number of ticks and labels to show below the X axis. */
    xTickCount: number;
    /** The number of smaller ticks to show between ticks on the x axis. */
    xSmallTickCount?: number;
    /** The number of lines and labels to show for the y axis. */
    yTickCount: number;
    /**
     * The list of parameters for which to draw lines.
     * These parameters belong to the main y axis on the left side.
     */
    primaryParameters: string[];
    /**
     * Secondary list of parameters for which to draw lines.
     * These parameters belong to the secondary y axis on the right side.
     *
     * It is recommended to only use `secondaryParameters` in addition to
     * `primaryParameters` and not on their own.
     */
    secondaryParameters: string[];
    /**
     * A map containing all parameters and the corresponding data.
     * Each entry in `primaryParameters` and `secondaryParameters`
     * should have a corresponding key in this map.
     */
    values: Record<string, ChartValue[]>;
    /**
     * Function that formats the timestamps to be displayed on the X axis.
     *
     * @param date  the date to format
     */
    formatDate(date: Date): string;
    /**
     * Function that formats the values to be displayed on the Y axis.
     *
     * @param value  the value to format
     * @param secondary  whether the label belongs to the secondary axis
     */
    formatValue(value: number, secondary: boolean): string;
    /** Parameter ids mapped to colors. */
    colors: Record<string, Colors>;
    /**
     * The time range inside which the values belong.
     * Format is [startTime, endTime].
     *
     * Timestamps should be given as unix timestamps (i.e. in seconds).
     */
    timestampRange: [number, number];
    /**
     * Override the range of the main y axis with this interval.
     * Passing this prop disables calculating the range from the values.
     */
    primaryRange?: [number, number];
    /**
     * Override the range of the secondary y axis with this interval.
     * Passing this prop disables calculating the range from the values.
     */
    secondaryRange?: [number, number];
} & ChartSelectionProps;
export declare const DoubleAxisLineChart: FC<DoubleAxisLineChartProps>;
//# sourceMappingURL=double-axis-line-chart.d.ts.map