"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCalendarListRef = exports.useCalendarRef = void 0;
const react_1 = __importStar(require("react"));
const moment_1 = __importDefault(require("moment"));
const calendar_list_item_1 = require("./calendar-list-item");
const calendar_settings_1 = require("./calendar-settings");
/**
 * Sets up the ref for `Calendar`-reliant components so it provides the
 * desired jump to date functionality.
 *
 * @param ref The reference to set up the imperative methods on.
 * @param selectedDay The currently selected day, formatted as a string.
 * @param onPress Callback to trigger with the date change.
 *
 * @returns A string representation of a date to use as `initialDate` to
 * ensure the correct month will be refocused.
 */
function useCalendarRef(ref, selectedDay, onPress) {
    const [monthIndicatorMoment, setMonthIndicatorMoment] = (0, react_1.useState)(() => (0, moment_1.default)(selectedDay));
    (0, react_1.useEffect)(() => {
        const selectedMoment = (0, moment_1.default)(selectedDay);
        if (!selectedMoment.isSame(monthIndicatorMoment, "day")) {
            // Keep month indicator and selectedDay in sync.
            setMonthIndicatorMoment(selectedMoment);
        }
    }, [monthIndicatorMoment, selectedDay]);
    (0, react_1.useImperativeHandle)(ref, () => ({
        jumpTo(date) {
            const selectedMoment = (0, moment_1.default)(selectedDay);
            const targetMoment = (0, moment_1.default)(date);
            if (selectedMoment.isSame(targetMoment, "week")) {
                /*
                If the selected day is the same as the date to jump to
                (or the start of the week for weekly resolution)
                then a change would not trigger when the user invokes this
                method and the displayed month would remain unchanged.
                So in this case it's necessary to select a different date
                from the same month that would trigger the change.
                */
                const dayAfterSelected = targetMoment
                    .clone()
                    .add(1, "week");
                const correctionMoment = dayAfterSelected.isSame(targetMoment, "month")
                    ? dayAfterSelected
                    : targetMoment.clone().subtract(1, "week");
                setMonthIndicatorMoment(correctionMoment);
            }
            else {
                setMonthIndicatorMoment(targetMoment);
            }
            onPress({ timestamp: targetMoment.unix() * 1000 });
        },
    }), [onPress, selectedDay]);
    return { initialMonth: monthIndicatorMoment.format(calendar_settings_1.CALENDAR_TIME_FORMAT) };
}
exports.useCalendarRef = useCalendarRef;
/**
 * Sets up the ref for Calendar List components so it provides the
 * desired jump to date functionality.
 *
 * @param ref The reference to set up the imperative methods on.
 * @param calendarList The data the list is displaying.
 * @param resolution Whether the list shows months or years.
 * @param onChange Callback to trigger with the date change.
 *
 * @returns A ref object to pass to the underlying `FlatList`.
 */
function useCalendarListRef(ref, calendarList, resolution, onChange) {
    // eslint-disable-next-line unicorn/no-null
    const calendarListRef = react_1.default.useRef(null);
    (0, react_1.useImperativeHandle)(ref, () => ({
        jumpTo(date) {
            var _a;
            const targetDate = (0, moment_1.default)(date);
            const targetIndex = calendarList.findIndex((item) => targetDate.isBetween(item.date, item.date.clone().endOf(resolution)));
            (_a = calendarListRef.current) === null || _a === void 0 ? void 0 : _a.scrollToOffset({
                offset: targetIndex * calendar_list_item_1.LIST_ITEM_HEIGHT,
            });
            onChange(targetDate.clone().startOf(resolution).toDate());
        },
    }), [calendarList, onChange, resolution]);
    return calendarListRef;
}
exports.useCalendarListRef = useCalendarListRef;
