/// <reference types="react" />
import { ListItemStyle } from "../../theme/themes";
import { TestProps } from "../../util";
export type DropdownOption = {
    label: string;
    disabled?: boolean;
};
export type CustomDropdownProps<T> = TestProps & {
    /** The value that is currently selected. */
    value: T | undefined;
    /**
     * Callback to be called when an option is pressed.
     *
     * @param option the pressed option
     */
    onOptionPress(option: T): void;
    /** Whether the screen is focused or not. Needed for rendering the options in a Modal. */
    screenInFocus: boolean;
    /** Component to be rendered as the header. */
    headerComponent: JSX.Element;
    /** Multiple `DropdownItem` components passed as children. */
    children: React.ReactNode;
    /** Disable interactions with the component. */
    disabled?: boolean;
};
export type DropdownProps = TestProps & Omit<CustomDropdownProps<string>, "headerComponent" | "children"> & {
    /** Label to display above the selected value. */
    label?: string;
    /** Placeholder for the value. This text will be rendered before an option has been selected. */
    placeholder?: string;
    /**
     * A map of values mapped to a label and optionally a disabled flag.
     */
    items: Record<string, DropdownOption>;
};
export type DropdownItemProps<T> = TestProps & {
    /** The value that the item represents. Take care to use a type that is compatible with the enclosing `CustomDropdown`'s value type. */
    value: T;
    /** Label to display. */
    label?: string;
    /** Disable interactions with the item. */
    disabled?: boolean;
    /** Custom component to be rendered instead of the label. The label will not be shown if this is given. */
    children?: React.ReactNode;
    /** Optional fill and line color for highlighting the selected values row. */
    selectedColor?: ListItemStyle;
};
export type InternalDropdownItemProps<T> = DropdownItemProps<T> & {
    /** The value that is currently selected. */
    currentValue: T | undefined;
    /**
     * Callback to be called when an option is pressed.
     *
     * @param option the pressed option
     */
    onPress(option: T): void;
};
//# sourceMappingURL=dropdown-props.d.ts.map