"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListSection = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const text_1 = require("../text");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    header: {
        flex: 1,
        paddingHorizontal: theme_1.METRICS.SCREEN.basePadding,
    },
    leftItem: {
        paddingLeft: theme_1.METRICS.CONTAINER.basePadding,
    },
    rightItem: {
        paddingRight: theme_1.METRICS.CONTAINER.basePadding,
    },
    wrapper: {
        alignItems: "center",
        flexDirection: "row",
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
        paddingTop: theme_1.METRICS.CONTAINER.doubleBasePadding,
    },
});
/**
 * Wraps multiple ListItems that belong in the same group and adds a header
 * with the section's name before the elements.
 *
 * @see ListItem
 */
const ListSection = ({ title, children, leftComponent, rightComponent, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    return (react_1.default.createElement(react_native_1.View, null,
        react_1.default.createElement(react_native_1.View, { style: [
                styles.wrapper,
                {
                    backgroundColor: theme.sectionBackgroundColor,
                },
            ], ...(0, util_1.getTestIdProps)(testID) },
            leftComponent && (react_1.default.createElement(react_native_1.View, { style: styles.leftItem }, leftComponent)),
            react_1.default.createElement(text_1.Title, { style: styles.header }, title),
            rightComponent && (react_1.default.createElement(react_native_1.View, { style: styles.rightItem }, rightComponent))),
        children));
};
exports.ListSection = ListSection;
