"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IosModal = void 0;
/* eslint-disable react/jsx-props-no-spreading */
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const containers_1 = require("../containers");
var IosColors;
(function (IosColors) {
    IosColors["GREY"] = "#d1d1d6";
    IosColors["DARK_GREY"] = "#636366";
    IosColors["BLUE"] = "#007aff";
    IosColors["LIGHT_BLUE"] = "#409cff";
})(IosColors || (IosColors = {}));
const styles = react_native_1.StyleSheet.create({
    button: {
        alignItems: "center",
        justifyContent: "center",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
    buttonContainer: {
        borderTopWidth: 1,
    },
    buttonText: {
        fontSize: theme_1.FontSize.MEDIUM,
        textAlign: "center",
    },
    container: {
        borderRadius: 15,
        maxHeight: "80%",
        maxWidth: "80%",
    },
    content: {
        alignItems: "center",
        padding: theme_1.METRICS.CONTAINER.doubleBasePadding,
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
    },
    doubleButton: { flexGrow: 1 },
    doubleButtonContainer: {
        borderTopWidth: 1,
        flexDirection: "row",
    },
    leftButtonContainer: {
        borderRightWidth: 1,
        flex: 1,
    },
    message: {
        fontSize: theme_1.FontSize.SMALL,
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
        textAlign: "center",
    },
    modal: {
        alignItems: "center",
        flex: 1,
        justifyContent: "center",
    },
    rightButtonContainer: {
        flex: 1,
    },
    title: {
        fontSize: theme_1.FontSize.MEDIUM,
        fontWeight: "600",
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
        textAlign: "center",
    },
    view: {
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
    },
});
const IosButton = ({ text, onPress, style, testID }) => {
    const { theme } = (0, theme_1.useTheme)();
    return (react_1.default.createElement(react_native_1.TouchableOpacity, { onPress: onPress, style: react_native_1.StyleSheet.flatten([styles.button, style]), ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_1.Text, { style: [
                styles.buttonText,
                {
                    color: theme.dark
                        ? IosColors.LIGHT_BLUE
                        : IosColors.BLUE,
                },
            ] }, text)));
};
const IosButtonSection = ({ buttons }) => {
    const { theme } = (0, theme_1.useTheme)();
    return buttons.length === 2 ? (react_1.default.createElement(react_native_1.View, { style: [
            styles.doubleButtonContainer,
            {
                borderColor: theme.dark
                    ? IosColors.DARK_GREY
                    : IosColors.GREY,
            },
        ] },
        react_1.default.createElement(react_native_1.View, { style: [
                styles.leftButtonContainer,
                {
                    borderColor: theme.dark
                        ? IosColors.DARK_GREY
                        : IosColors.GREY,
                },
            ] },
            react_1.default.createElement(IosButton, { style: styles.doubleButton, ...buttons[0] })),
        react_1.default.createElement(react_native_1.View, { style: styles.rightButtonContainer },
            react_1.default.createElement(IosButton, { style: styles.doubleButton, ...buttons[1] })))) : (react_1.default.createElement(react_native_1.View, null, buttons.map((buttonProps) => (react_1.default.createElement(react_native_1.View, { key: buttonProps.text, style: [
            styles.buttonContainer,
            {
                borderColor: theme.dark
                    ? IosColors.DARK_GREY
                    : IosColors.GREY,
            },
        ] },
        react_1.default.createElement(IosButton, { ...buttonProps }))))));
};
const IosModal = ({ visible, transparent = true, title, message, buttons, children, testID, withInput = false, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const modalBody = (react_1.default.createElement(react_native_1.View, { style: [
            styles.modal,
            { backgroundColor: theme.modalBackgroundColor },
        ] },
        react_1.default.createElement(react_native_1.View, { style: [
                styles.container,
                {
                    backgroundColor: theme.backgroundColor,
                },
            ], ...(0, util_1.getTestIdProps)(testID) },
            react_1.default.createElement(react_native_1.View, { style: styles.content },
                title !== undefined && (react_1.default.createElement(react_native_1.Text, { style: [styles.title, { color: theme.textColor }] }, title)),
                message !== undefined && (react_1.default.createElement(react_native_1.Text, { style: [styles.message, { color: theme.textColor }] }, message)),
                children && react_1.default.createElement(react_native_1.View, { style: styles.view }, children)),
            buttons && react_1.default.createElement(IosButtonSection, { buttons: buttons }))));
    return (react_1.default.createElement(react_native_1.Modal, { transparent: transparent, visible: visible, ...(0, util_1.getTestIdProps)(`${testID}.modal`) }, withInput ? (react_1.default.createElement(containers_1.KeyboardContainer, null, modalBody)) : (modalBody)));
};
exports.IosModal = IosModal;
