"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessIndicatorWithManySteps = void 0;
const lodash_1 = require("lodash");
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const react_native_svg_1 = __importStar(require("react-native-svg"));
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    indicatorWrapper: {
        flexDirection: "row",
        justifyContent: "center",
        margin: theme_1.METRICS.CONTAINER.baseMargin,
    },
});
const SIZES = {
    radius: (0, theme_1.verticalScale)(8),
    activeRadius: (0, theme_1.verticalScale)(14.5),
    stroke: 1.5,
    lineWidth: (0, theme_1.horizontalScale)(21),
    fontSize: (0, theme_1.verticalScale)(15),
};
const SHORTER_LINE_SIZE = SIZES.lineWidth - (SIZES.activeRadius - SIZES.radius);
const CURRENT_STAGE_SIZE = (SIZES.activeRadius + SIZES.stroke) * 2;
const DOT_CIRCLE_SIZE = (SIZES.radius + SIZES.stroke) * 2 - SIZES.stroke / 2;
const SCREEN_WIDTH = react_native_1.Dimensions.get("window").width;
var StageType;
(function (StageType) {
    StageType[StageType["COMPLETED"] = 0] = "COMPLETED";
    StageType[StageType["CURRENT"] = 1] = "CURRENT";
    StageType[StageType["REMAINING"] = 2] = "REMAINING";
})(StageType || (StageType = {}));
const getStageType = (currentStage, stage) => {
    if (stage < currentStage) {
        return StageType.COMPLETED;
    }
    if (stage > currentStage) {
        return StageType.REMAINING;
    }
    return StageType.CURRENT;
};
const getRenderedPartSize = (stage, currentStage) => {
    const prevCount = Math.max(0, stage - 2);
    const prevStageCount = Math.max(0, stage - 1);
    if (stage - currentStage === 1) {
        return (prevCount * DOT_CIRCLE_SIZE +
            Math.max(0, prevCount - 1) * SIZES.lineWidth +
            (currentStage > 1 ? 1 : 0) * SHORTER_LINE_SIZE +
            CURRENT_STAGE_SIZE -
            SIZES.stroke / 2);
    }
    return prevStageCount * DOT_CIRCLE_SIZE + prevCount * SIZES.lineWidth;
};
const renderStage = (currentStage, stage, maxStages, theme) => {
    const stageType = getStageType(currentStage, stage);
    const strokeColor = stageType === StageType.REMAINING
        ? theme.disabledColor
        : theme.primaryColor;
    let lineWidth = stageType !== StageType.COMPLETED && stage - currentStage < 2
        ? SHORTER_LINE_SIZE
        : SIZES.lineWidth;
    if ((stage === maxStages && stage === currentStage) ||
        (currentStage === 1 && stage === 2)) {
        lineWidth -= SIZES.activeRadius - SIZES.radius;
    }
    return (react_1.default.createElement(react_native_svg_1.G, { key: stage, x: getRenderedPartSize(stage, currentStage) },
        stage > 1 && (react_1.default.createElement(react_native_svg_1.Line, { stroke: strokeColor, strokeWidth: SIZES.stroke, x2: lineWidth + SIZES.stroke / 2, y: SIZES.activeRadius + SIZES.stroke })),
        react_1.default.createElement(react_native_svg_1.G, { x: stage > 1 ? lineWidth : 0 },
            stageType === StageType.CURRENT && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(react_native_svg_1.Circle, { cx: SIZES.activeRadius + SIZES.stroke, cy: SIZES.activeRadius + SIZES.stroke, fill: theme.backgroundColor, r: SIZES.activeRadius, stroke: strokeColor, strokeWidth: SIZES.stroke }),
                react_1.default.createElement(react_native_svg_1.Text, { alignmentBaseline: "central", fill: strokeColor, fontFamily: theme.regularFont, fontSize: SIZES.fontSize, fontWeight: 500, textAnchor: "middle", x: CURRENT_STAGE_SIZE / 2, y: CURRENT_STAGE_SIZE / 2 }, stage))),
            stageType !== StageType.CURRENT && (react_1.default.createElement(react_native_svg_1.Circle, { cx: SIZES.radius + SIZES.stroke, cy: SIZES.activeRadius + SIZES.stroke, fill: strokeColor, r: SIZES.radius, stroke: strokeColor, strokeWidth: SIZES.stroke })))));
};
/**
 * Component used for displaying the status of a process in case of more than 7 stages.
 * Stages are represented as dots and the current stage
 * has its index in the center of a circle.
 */
const ProcessIndicatorWithManySteps = ({ maxStages, currentStage, style, maxWidth = SCREEN_WIDTH - theme_1.METRICS.SCREEN.basePadding * 2, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const stages = (0, lodash_1.range)(1, maxStages + 1);
    const width = DOT_CIRCLE_SIZE * maxStages + SIZES.lineWidth * (maxStages - 1);
    const svgWidth = Math.min(width, maxWidth);
    const resultingStyle = react_native_1.StyleSheet.flatten([styles.indicatorWrapper, style]);
    return (
    // eslint-disable-next-line react/jsx-props-no-spreading
    react_1.default.createElement(react_native_1.View, { style: resultingStyle, ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_svg_1.default, { height: CURRENT_STAGE_SIZE, viewBox: `0 0 ${width} ${CURRENT_STAGE_SIZE}`, width: svgWidth }, stages.map((stage) => renderStage(currentStage, stage, maxStages, theme)))));
};
exports.ProcessIndicatorWithManySteps = ProcessIndicatorWithManySteps;
