"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionCircle = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const icon_size_1 = require("../icons/icon-size");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    circle: {
        alignItems: "center",
        borderRadius: icon_size_1.IconSize.XSMALL / 2,
        height: icon_size_1.IconSize.XSMALL,
        justifyContent: "center",
        width: icon_size_1.IconSize.XSMALL,
    },
    circleCenter: {
        borderRadius: icon_size_1.IconSize.XSMALL / 4,
        height: icon_size_1.IconSize.XSMALL / 2,
        width: icon_size_1.IconSize.XSMALL / 2,
    },
});
/**
 * Selection circle to be used for indicating selection in a list or for
 * radio buttons.
 */
const SelectionCircle = ({ selected, pressed = false, disabled = false, style, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const variant = selected
        ? theme.radioButton.selected
        : theme.radioButton.unselected;
    let colors = variant.default;
    if (disabled) {
        colors = variant.disabled;
    }
    else if (pressed) {
        colors = variant.pressed;
    }
    const buttonBackgroundColor = colors.fillColor;
    const buttonBorderColor = colors.borderColor;
    const buttonCenterColor = disabled
        ? theme.radioButton.disabledCenterColor
        : pressed
            ? theme.radioButton.pressedCenterColor
            : theme.radioButton.centerColor;
    return (react_1.default.createElement(react_native_1.View, { style: [
            styles.circle,
            {
                backgroundColor: buttonBackgroundColor,
                borderColor: buttonBorderColor,
                borderWidth: theme.radioButton.borderWidth,
            },
            style,
        ], ...(0, util_1.getTestIdProps)(testID) }, selected && (react_1.default.createElement(react_native_1.View, { style: [
            styles.circleCenter,
            {
                backgroundColor: buttonCenterColor,
            },
        ] }))));
};
exports.SelectionCircle = SelectionCircle;
