"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TileContainer = void 0;
const react_1 = __importDefault(require("react"));
const lodash_1 = require("lodash");
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const styles = react_native_1.StyleSheet.create({
    margin: {
        marginRight: theme_1.METRICS.SCREEN.basePadding,
    },
    squareView: {
        aspectRatio: 1,
        flex: 1,
    },
    tileRow: {
        flexDirection: "row",
    },
    tileRowWithMargin: {
        flexDirection: "row",
        paddingTop: theme_1.METRICS.SCREEN.basePadding,
    },
});
/**
 * A component that renders the tiles it receives as children
 * two on a row, with a gap between them.
 */
const TileContainer = ({ children }) => {
    const childrenWithMargins = react_1.default.Children.toArray(children).map(addMarginToEvenChild);
    const evenChildren = makeChildrenEven(childrenWithMargins);
    const pairedTiles = (0, lodash_1.chunk)(evenChildren, 2);
    return (
    // eslint-disable-next-line react/jsx-no-useless-fragment
    react_1.default.createElement(react_1.default.Fragment, null, pairedTiles.map((tiles, i) => (react_1.default.createElement(react_native_1.View
    // eslint-disable-next-line react/no-array-index-key
    , { 
        // eslint-disable-next-line react/no-array-index-key
        key: i, style: i !== 0 ? styles.tileRowWithMargin : styles.tileRow }, tiles)))));
};
exports.TileContainer = TileContainer;
function addMarginToEvenChild(child, index) {
    const isEven = index % 2 === 0;
    if (!isEven) {
        return child;
    }
    return react_1.default.cloneElement(child, {
        style: styles.margin,
    });
}
function makeChildrenEven(children) {
    const isEven = children.length % 2 === 0;
    if (isEven) {
        return children;
    }
    const squareView = (react_1.default.createElement(react_native_1.View, { key: "ending-square-view", style: styles.squareView }));
    return [...children, squareView];
}
