"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buderusTheme = exports.boschTheme = void 0;
const colors_1 = require("./colors");
const fonts_1 = require("./fonts");
const icon_size_1 = require("../components/icons/icon-size");
const scaling_1 = require("./scaling");
exports.boschTheme = {
    textColor: colors_1.Colors.BLACK,
    secondaryTextColor: colors_1.Colors.WHITE,
    backgroundColor: colors_1.Colors.WHITE,
    secondaryBackgroundColor: colors_1.Colors.BOSCH_GRAY_90,
    primaryColor: colors_1.Colors.BOSCH_BLUE_50,
    pressedColor: colors_1.Colors.BOSCH_BLUE_30,
    secondaryPressedColor: colors_1.Colors.BOSCH_BLUE_80,
    disabledColor: colors_1.Colors.BOSCH_GRAY_80,
    disabledTextColor: colors_1.Colors.BOSCH_GRAY_60,
    regularFont: fonts_1.FontFaces.REGULAR,
    boldFont: fonts_1.FontFaces.BOLD,
    allowFontScaling: true,
    dark: false,
    statusBarStyle: "dark-content",
    keyboardAppearance: "light",
    coloredLoadingIndicator: true,
    showSupergraphic: true,
    separatorColor: colors_1.Colors.BOSCH_GRAY_60,
    separatorSize: 1,
    modalBackgroundColor: colors_1.Colors.BOSCH_GRAY_40_TRANSPARENT,
    hintLabelColor: colors_1.Colors.BOSCH_GRAY_40,
    sectionBackgroundColor: colors_1.Colors.BOSCH_GRAY_90,
    listSecondaryColor: colors_1.Colors.BLACK,
    tileTextColor: colors_1.Colors.WHITE,
    activeOpacity: 0.5,
    error: {
        fillColor: colors_1.Colors.BOSCH_RED_90,
        lineColor: colors_1.Colors.BOSCH_RED_50,
        textColor: colors_1.Colors.BOSCH_RED_40,
    },
    warning: {
        fillColor: colors_1.Colors.BOSCH_YELLOW_90,
        lineColor: colors_1.Colors.BOSCH_YELLOW_85,
        textColor: colors_1.Colors.BOSCH_YELLOW_40,
    },
    success: {
        fillColor: colors_1.Colors.BOSCH_GREEN_90,
        lineColor: colors_1.Colors.BOSCH_GREEN_60,
        textColor: colors_1.Colors.BOSCH_GREEN_40,
    },
    info: {
        fillColor: colors_1.Colors.BOSCH_BLUE_90,
        lineColor: colors_1.Colors.BOSCH_BLUE_50,
        textColor: colors_1.Colors.BOSCH_BLUE_40,
    },
    checkbox: {
        borderWidth: 0,
        iconSize: (0, scaling_1.verticalScale)(22),
        coloredTickColor: colors_1.Colors.WHITE,
        tickColor: colors_1.Colors.WHITE,
        disabledTickColor: colors_1.Colors.BOSCH_GRAY_60,
        checked: {
            default: {
                fillColor: colors_1.Colors.BOSCH_BLUE_50,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_BLUE_30,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_80,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
        },
        unchecked: {
            default: {
                fillColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_GRAY_40,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_90,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
        },
    },
    inputs: {
        placeholderColor: colors_1.Colors.BOSCH_GRAY_40,
        selectionColor: {
            ios: colors_1.Colors.BOSCH_BLUE_50,
            android: colors_1.Colors.BOSCH_BLUE_80,
        },
        default: {
            fillColor: colors_1.Colors.BOSCH_GRAY_90,
            lineColor: colors_1.Colors.BLACK,
            textColor: colors_1.Colors.BLACK,
        },
        focused: {
            fillColor: colors_1.Colors.BOSCH_BLUE_90,
            lineColor: colors_1.Colors.BOSCH_BLUE_50,
            textColor: colors_1.Colors.BLACK,
        },
        pressed: {
            fillColor: colors_1.Colors.BOSCH_GRAY_70,
            lineColor: colors_1.Colors.BLACK,
            textColor: colors_1.Colors.BLACK,
        },
        disabled: {
            fillColor: colors_1.Colors.BOSCH_GRAY_90,
            lineColor: colors_1.Colors.BOSCH_GRAY_70,
            textColor: colors_1.Colors.BOSCH_GRAY_70,
        },
    },
    list: {
        default: {
            fillColor: colors_1.Colors.WHITE,
            lineColor: colors_1.Colors.BOSCH_GRAY_80,
        },
        pressed: {
            fillColor: colors_1.Colors.BOSCH_GRAY_80,
            lineColor: colors_1.Colors.BOSCH_GRAY_60,
        },
        disabled: {
            fillColor: colors_1.Colors.WHITE,
            lineColor: colors_1.Colors.BOSCH_GRAY_80,
        },
    },
    nativeSwitch: {
        enabledOn: {
            trackColor: colors_1.Colors.BOSCH_BLUE_50,
            borderColor: colors_1.Colors.TRANSPARENT,
            thumbColor: colors_1.Colors.WHITE,
        },
        ios: {
            enabledOff: {
                trackColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.TRANSPARENT,
                thumbColor: colors_1.Colors.WHITE,
            },
            disabled: {
                trackColor: colors_1.Colors.BOSCH_GRAY_90,
                borderColor: colors_1.Colors.TRANSPARENT,
                thumbColor: colors_1.Colors.BOSCH_GRAY_60,
            },
        },
        android: {
            enabledOff: {
                trackColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.TRANSPARENT,
                thumbColor: colors_1.Colors.WHITE,
            },
            disabled: {
                trackColor: colors_1.Colors.BOSCH_GRAY_90,
                borderColor: colors_1.Colors.TRANSPARENT,
                thumbColor: colors_1.Colors.BOSCH_GRAY_60,
            },
        },
    },
    progressBar: {
        trackColor: colors_1.Colors.BOSCH_GRAY_90,
    },
    radioButton: {
        borderWidth: 0,
        centerColor: colors_1.Colors.WHITE,
        pressedCenterColor: colors_1.Colors.WHITE,
        disabledCenterColor: colors_1.Colors.BOSCH_GRAY_60,
        selected: {
            default: {
                fillColor: colors_1.Colors.BOSCH_BLUE_50,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_BLUE_30,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_80,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
        },
        unselected: {
            default: {
                fillColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_GRAY_40,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_90,
                borderColor: colors_1.Colors.TRANSPARENT,
            },
        },
    },
    segmentedControl: {
        fontSize: fonts_1.FontSize.MEDIUM,
        iconSize: icon_size_1.IconSize.SMALL,
        selected: {
            regular: {
                backgroundColor: colors_1.Colors.BOSCH_BLUE_50,
                contentColor: colors_1.Colors.WHITE,
                borderColor: colors_1.Colors.WHITE,
            },
            disabled: {
                backgroundColor: colors_1.Colors.BOSCH_GRAY_80,
                contentColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                backgroundColor: colors_1.Colors.BOSCH_BLUE_30,
                contentColor: colors_1.Colors.WHITE,
                borderColor: colors_1.Colors.WHITE,
            },
        },
        tab: {
            regular: {
                backgroundColor: colors_1.Colors.BOSCH_GRAY_90,
                contentColor: colors_1.Colors.BLACK,
                borderColor: colors_1.Colors.WHITE,
            },
            disabled: {
                backgroundColor: colors_1.Colors.BOSCH_GRAY_90,
                contentColor: colors_1.Colors.BOSCH_GRAY_70,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                backgroundColor: colors_1.Colors.BOSCH_GRAY_70,
                contentColor: colors_1.Colors.WHITE,
                borderColor: colors_1.Colors.WHITE,
            },
        },
    },
    slider: {
        default: {
            backgroundColor: colors_1.Colors.BOSCH_GRAY_60,
            color: colors_1.Colors.BOSCH_BLUE_50,
        },
        pressed: {
            backgroundColor: colors_1.Colors.BOSCH_GRAY_40,
            color: colors_1.Colors.BOSCH_BLUE_30,
        },
        disabled: {
            backgroundColor: colors_1.Colors.BOSCH_GRAY_90,
            color: colors_1.Colors.BOSCH_GRAY_60,
        },
    },
    switch: {
        borderWidth: 0,
        on: {
            default: {
                fillColor: colors_1.Colors.BOSCH_BLUE_50,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_BLUE_30,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.WHITE,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_80,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.BOSCH_GRAY_60,
            },
        },
        off: {
            default: {
                fillColor: colors_1.Colors.BOSCH_GRAY_60,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BOSCH_GRAY_40,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.WHITE,
            },
            disabled: {
                fillColor: colors_1.Colors.BOSCH_GRAY_90,
                borderColor: colors_1.Colors.TRANSPARENT,
                centerColor: colors_1.Colors.BOSCH_GRAY_70,
            },
        },
    },
    whatsNew: {
        backgroundColor: colors_1.Colors.BLACK_TRANSPARENT_DARK,
        textColor: colors_1.Colors.WHITE,
        pressedColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
        secondaryPressedColor: colors_1.Colors.TRANSPARENT,
    },
    dropdown: {
        backgroundColor: colors_1.Colors.BOSCH_GRAY_90,
        disabledColor: colors_1.Colors.BOSCH_GRAY_70,
        optionBackgroundColor: colors_1.Colors.WHITE,
        optionPressedBackgroundColor: colors_1.Colors.BOSCH_GRAY_80,
        optionSelectedBackgroundColor: colors_1.Colors.BOSCH_BLUE_50,
    },
};
exports.buderusTheme = {
    textColor: colors_1.Colors.WHITE,
    secondaryTextColor: colors_1.Colors.BUDERUS_DARK_GREY,
    backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY,
    secondaryBackgroundColor: colors_1.Colors.BUDERUS_DARK_GREY,
    primaryColor: colors_1.Colors.WHITE,
    pressedColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
    secondaryPressedColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
    disabledColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
    disabledTextColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
    regularFont: fonts_1.FontFaces.REGULAR,
    boldFont: fonts_1.FontFaces.BOLD,
    allowFontScaling: true,
    dark: true,
    statusBarStyle: "light-content",
    keyboardAppearance: "dark",
    coloredLoadingIndicator: false,
    showSupergraphic: false,
    separatorColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
    separatorSize: 1,
    modalBackgroundColor: colors_1.Colors.BLACK_TRANSPARENT,
    hintLabelColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
    sectionBackgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
    listSecondaryColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
    tileTextColor: colors_1.Colors.WHITE,
    activeOpacity: 0.5,
    error: {
        fillColor: colors_1.Colors.BUDERUS_RED,
        lineColor: colors_1.Colors.BUDERUS_RED,
        textColor: colors_1.Colors.WHITE,
    },
    warning: {
        fillColor: colors_1.Colors.BUDERUS_YELLOW,
        lineColor: colors_1.Colors.BUDERUS_YELLOW,
        textColor: colors_1.Colors.BUDERUS_DARK_GREY,
    },
    success: {
        fillColor: colors_1.Colors.BUDERUS_GREEN,
        lineColor: colors_1.Colors.BUDERUS_GREEN,
        textColor: colors_1.Colors.WHITE,
    },
    info: {
        fillColor: colors_1.Colors.WHITE,
        lineColor: colors_1.Colors.BUDERUS_LIGHT_BLUE,
        textColor: colors_1.Colors.BUDERUS_DARK_GREY,
    },
    checkbox: {
        borderWidth: 2,
        iconSize: (0, scaling_1.verticalScale)(22),
        coloredTickColor: colors_1.Colors.WHITE,
        tickColor: colors_1.Colors.BUDERUS_DARK_GREY,
        disabledTickColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        checked: {
            default: {
                fillColor: colors_1.Colors.WHITE,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
        unchecked: {
            default: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
    },
    inputs: {
        placeholderColor: colors_1.Colors.BUDERUS_DARK_GREY_W75,
        selectionColor: {
            ios: colors_1.Colors.WHITE,
            android: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
        },
        default: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
            lineColor: colors_1.Colors.WHITE,
            textColor: colors_1.Colors.WHITE,
        },
        focused: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            lineColor: colors_1.Colors.WHITE,
            textColor: colors_1.Colors.WHITE,
        },
        pressed: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            lineColor: colors_1.Colors.WHITE,
            textColor: colors_1.Colors.WHITE,
        },
        disabled: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
            lineColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            textColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        },
    },
    list: {
        default: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
            lineColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        },
        pressed: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            lineColor: colors_1.Colors.BUDERUS_DARK_GREY_W75,
        },
        disabled: {
            fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
            lineColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        },
    },
    nativeSwitch: {
        enabledOn: {
            trackColor: colors_1.Colors.BUDERUS_DARK_GREY_W75,
            borderColor: colors_1.Colors.WHITE,
            thumbColor: colors_1.Colors.WHITE,
        },
        ios: {
            enabledOff: {
                trackColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
                thumbColor: colors_1.Colors.WHITE,
            },
            disabled: {
                trackColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                thumbColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
        android: {
            enabledOff: {
                trackColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.WHITE,
                thumbColor: colors_1.Colors.WHITE,
            },
            disabled: {
                trackColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                thumbColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
    },
    progressBar: {
        trackColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
    },
    radioButton: {
        borderWidth: icon_size_1.IconSize.XSMALL / 8,
        centerColor: colors_1.Colors.WHITE,
        pressedCenterColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
        disabledCenterColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        selected: {
            default: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
        unselected: {
            default: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
    },
    segmentedControl: {
        fontSize: fonts_1.FontSize.MEDIUM,
        iconSize: icon_size_1.IconSize.SMALL,
        selected: {
            regular: {
                backgroundColor: colors_1.Colors.WHITE,
                contentColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
            },
            disabled: {
                backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                contentColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
            pressed: {
                backgroundColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                contentColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
        },
        tab: {
            regular: {
                backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY,
                contentColor: colors_1.Colors.WHITE,
                borderColor: colors_1.Colors.WHITE,
            },
            disabled: {
                backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY,
                contentColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
            pressed: {
                backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                contentColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
        },
    },
    slider: {
        default: {
            backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            color: colors_1.Colors.WHITE,
        },
        pressed: {
            backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
            color: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
        },
        disabled: {
            backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
            color: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        },
    },
    switch: {
        borderWidth: icon_size_1.IconSize.XSMALL / 8,
        on: {
            default: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W75,
                borderColor: colors_1.Colors.WHITE,
                centerColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                centerColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                centerColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
        off: {
            default: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.WHITE,
                centerColor: colors_1.Colors.WHITE,
            },
            pressed: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
                borderColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
                centerColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
            },
            disabled: {
                fillColor: colors_1.Colors.BUDERUS_DARK_GREY,
                borderColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
                centerColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
            },
        },
    },
    whatsNew: {
        backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_TRANSPARENT_DARK,
        textColor: colors_1.Colors.WHITE,
        pressedColor: colors_1.Colors.BUDERUS_LIGHT_GREY_B50,
        secondaryPressedColor: colors_1.Colors.TRANSPARENT,
    },
    dropdown: {
        backgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        disabledColor: colors_1.Colors.BUDERUS_DARK_GREY_W75,
        optionBackgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W50,
        optionPressedBackgroundColor: colors_1.Colors.BUDERUS_DARK_GREY_W25,
        optionSelectedBackgroundColor: colors_1.Colors.WHITE,
    },
};
