//
//  Assembler.m
//  RNPrivacyComponent
//
//  Created by István Bege on 01/11/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//

#import "Assembler.h"
#import <UIKit/UIKit.h>

@implementation Assembler

+ (NSDictionary*)createCountryDTO:(TTPCountry*)country {
    return [[NSMutableDictionary alloc] initWithObjectsAndKeys:[country name], @"name", [country language], @"language", [country code], @"code", @"", @"flagRes", nil];
}

+ (TTPCountry*)createCountry:(NSDictionary*)dtoCountry {
    UIImage* flag = [UIImage imageNamed:dtoCountry[@"flagRes"]];
    return [[TTPCountry alloc] initWithCode:dtoCountry[@"code"] name:dtoCountry[@"name"] language:dtoCountry[@"language"] flag:flag];
}

+ (NSArray<TTPCountry*>*)createCountryArray:(NSArray<NSDictionary*>*)countryDTOArray {
    NSMutableArray<TTPCountry*>* countries = [NSMutableArray array];
    for (id rawCountry in countryDTOArray) {
        [countries addObject:[self createCountry:rawCountry]];
    }
    return countries;
}

+ (NSArray<NSDictionary*>*)createCountryDTOArray:(NSArray<TTPCountry*>*)countries {
    NSMutableArray<NSDictionary*>* dtoCountries = [NSMutableArray array];
    for (id country in countries) {
        [dtoCountries addObject:[self createCountryDTO:country]];
    }
    return dtoCountries;
}

+ (NSDictionary*)createDocumentDTO:(TTPMetaDocument*)document {
    return [[NSMutableDictionary alloc] initWithObjectsAndKeys:[document documentId], @"id", [document country], @"country", [document language], @"language", [document validFrom], @"validFrom", [document createdOn], @"createdOn", nil];
}

+ (NSArray<NSDictionary*>*)createDocumentDTOArray:(NSArray<TTPMetaDocument*>*)documents {
    NSMutableArray<NSDictionary*>* documentDTOArray = [NSMutableArray array];
    for (id document in documents) {
        [documentDTOArray addObject:[self createDocumentDTO:document]];
    }
    return documentDTOArray;
}

+ (NSDictionary*)createActivityDocumentDTO:(TTPActivityDocument*)document {
    return [[NSMutableDictionary alloc] initWithObjectsAndKeys:[document activityId], @"activityId", [document name], @"name", [NSNumber numberWithUnsignedInteger:[document revision]], @"revision", [document country], @"country", [document language], @"language", [document validFrom], @"validFrom", [document timestamp], @"timestamp", nil];
}

+ (NSArray<NSDictionary*>*)createActivityDocumentDTOArray:(NSArray<TTPMetaActivityDocument*>*)documents {
    NSMutableArray<NSDictionary*>* documentDTOArray = [NSMutableArray array];
    for (id document in documents) {
        [documentDTOArray addObject:[self createActivityDocumentDTO:document]];
    }
    return documentDTOArray;
}

@end
